@@ -1,6 +1,7 @@
 ;;; hippie-exp.el --- expand text trying various ways to find its expansion
 
-;; Copyright (C) 1992 Free Software Foundation, Inc.
+;; Copyright (C) 1992, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
+;;   2009, 2010, 2011  Free Software Foundation, Inc.
 
 ;; Author: Anders Holst <aho@sans.kth.se>
 ;; Last change: 3 March 1998
@@ -9,10 +10,10 @@
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is free software; you can redistribute it and/or modify
+;; GNU Emacs is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,15 +21,13 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
 ;;  `hippie-expand' is a single function for a lot of different kinds
 ;;  of completions and expansions.  Called repeatedly it tries all
-;;  possible completions in succession. 
+;;  possible completions in succession.
 ;;  Which kinds of completions to try, and in which order, is
 ;;  determined by the contents of `hippie-expand-try-functions-list'.
 ;;  Much customization of `hippie-expand' can be made by changing the
@@ -61,7 +60,7 @@
 ;;  characters of syntax '_' is considered part of the words to expand
 ;;  dynamically.
 ;;  See also the macro `make-hippie-expand-function' below.
-;;  
+;;
 ;;  A short description of the current try-functions in this file:
 ;;    `try-complete-file-name' : very convenient to have in any buffer,
 ;;      and not just in the minibuffer or (some) shell-mode.  It goes
@@ -72,25 +71,25 @@
 ;;      a file name completed only as many characters as is unique.
 ;;    `try-expand-all-abbrevs' : can be removed if you don't use abbrevs.
 ;;      Otherwise it looks through all abbrev-tables, starting with
-;;      the local followed by the global. 
-;;    `try-expand-line' : Searches the buffer for an entire line that 
-;;      begins exactly as the current line.  Convenient sometimes, for 
+;;      the local followed by the global.
+;;    `try-expand-line' : Searches the buffer for an entire line that
+;;      begins exactly as the current line.  Convenient sometimes, for
 ;;      example as a substitute for (or complement to) the history
 ;;      list in shell-like buffers.  At other times, only confusing.
 ;;    `try-expand-line-all-buffers' : Like `try-expand-line' but searches
 ;;      in all buffers (except the current).  (This may be a little
 ;;      slow, don't use it unless you are really fond of `hippie-expand'.)
 ;;    `try-expand-list' : Tries to expand the text back to the nearest
-;;      open delimiter, to a whole list from the buffer. Convenient for
-;;      example when writing lisp or TeX.
-;;    `try-expand-list-all-buffers' : Like `try-expand-list' but searches 
-;;      in all buffers (except the current).  
+;;      open delimiter, to a whole list from the buffer.  Convenient for
+;;      example when writing Lisp or TeX.
+;;    `try-expand-list-all-buffers' : Like `try-expand-list' but searches
+;;      in all buffers (except the current).
 ;;    `try-expand-dabbrev' : works exactly as dabbrev-expand (but of
 ;;      course in a way compatible with the other try-functions).
 ;;    `try-expand-dabbrev-all-buffers' : perhaps the most useful of them,
 ;;      like `dabbrev-expand' but searches all Emacs buffers (except the
 ;;      current) for matching words.  (No, I don't find this one
-;;      particularly slow.) 
+;;      particularly slow.)
 ;;    `try-expand-dabbrev-visible': Searches the currently visible parts of
 ;;      all windows.  Can be put before `try-expand-dabbrev-all-buffers' to
 ;;      first try the expansions you can see.
@@ -108,7 +107,7 @@
 ;;      already has a key of its own, you might want to remove this.
 ;;    `try-complete-lisp-symbol-partially' : To insert in the list just
 ;;      before `try-complete-lisp-symbol' for those who first want to get
-;;      completion of what is unique in the name.  
+;;      completion of what is unique in the name.
 ;;
 ;;  Not all of the above functions are by default in
 ;;  `hippie-expand-try-functions-list'.  This variable is better set
@@ -117,7 +116,7 @@
 ;;  variable with all kinds of try-functions above, it might be an
 ;;  idea to use `make-hippie-expand-function' to construct different
 ;;  `hippie-expand'-like functions, with different try-lists and bound
-;;  to different keys. It is also possible to make
+;;  to different keys.  It is also possible to make
 ;;  `hippie-expand-try-functions-list' a buffer local variable, and
 ;;  let it depend on the mode (by setting it in the mode-hooks).
 ;;
@@ -143,14 +142,14 @@
 ;;    `he-reset-string' : Resets the initialized region to its original
 ;;      contents.
 ;;  There is also a variable: `he-tried-table' which is meant to contain
-;;  all tried expansions so far.  The try-function can check this 
+;;  all tried expansions so far.  The try-function can check this
 ;;  variable to see whether an expansion has already been tried
 ;;  (hint: `he-string-member').
 ;;
 ;;  Known bugs
 ;;
 ;;  It may happen that some completion suggestion occurs twice, in
-;;  spite of the use of `he-tried-table' to prevent that.  This is 
+;;  spite of the use of `he-tried-table' to prevent that.  This is
 ;;  because different try-functions may try to complete different
 ;;  lengths of text, and thus put different amounts of the
 ;;  text in `he-tried-table'.  Anyway this seems to occur seldom enough
@@ -168,7 +167,7 @@
 
 ;;; Code:
 
-(eval-when-compile (require 'comint))
+(require 'comint)
 
 (defgroup hippie-expand nil
   "Expand text trying various ways to find its expansion."
@@ -221,39 +220,39 @@ or insert functions in this list."
 
 ;;;###autoload
 (defcustom hippie-expand-verbose t
-  "*Non-nil makes `hippie-expand' output which function it is trying."
+  "Non-nil makes `hippie-expand' output which function it is trying."
   :type 'boolean
   :group 'hippie-expand)
 
 ;;;###autoload
 (defcustom hippie-expand-dabbrev-skip-space nil
-  "*Non-nil means tolerate trailing spaces in the abbreviation to expand."
+  "Non-nil means tolerate trailing spaces in the abbreviation to expand."
   :group 'hippie-expand
   :type 'boolean)
 
 ;;;###autoload
 (defcustom hippie-expand-dabbrev-as-symbol t
-  "*Non-nil means expand as symbols, i.e. syntax `_' is considered a letter."
+  "Non-nil means expand as symbols, i.e. syntax `_' is considered a letter."
   :group 'hippie-expand
   :type 'boolean)
 
 ;;;###autoload
 (defcustom hippie-expand-no-restriction t
-  "*Non-nil means that narrowed buffers are widened during search."
+  "Non-nil means that narrowed buffers are widened during search."
   :group 'hippie-expand
   :type 'boolean)
 
 ;;;###autoload
 (defcustom hippie-expand-max-buffers ()
-  "*The maximum number of buffers (apart from the current) searched.
+  "The maximum number of buffers (apart from the current) searched.
 If nil, all buffers are searched."
   :type '(choice (const :tag "All" nil)
 		 integer)
   :group 'hippie-expand)
 
 ;;;###autoload
-(defcustom hippie-expand-ignore-buffers '("^ \\*.*\\*$" dired-mode)
-  "*A list specifying which buffers not to search (if not current).
+(defcustom hippie-expand-ignore-buffers (list (purecopy "^ \\*.*\\*$") 'dired-mode)
+  "A list specifying which buffers not to search (if not current).
 Can contain both regexps matching buffer names (as strings) and major modes
 \(as atoms)"
   :type '(repeat (choice regexp (symbol :tag "Major Mode")))
@@ -261,9 +260,9 @@ Can contain both regexps matching buffer names (as strings) and major modes
 
 ;;;###autoload
 (defcustom hippie-expand-only-buffers ()
-  "*A list specifying the only buffers to search (in addition to current).
+  "A list specifying the only buffers to search (in addition to current).
 Can contain both regexps matching buffer names (as strings) and major modes
-\(as atoms). If non-NIL, this variable overrides the variable
+\(as atoms).  If non-nil, this variable overrides the variable
 `hippie-expand-ignore-buffers'."
   :type '(repeat (choice regexp (symbol :tag "Major Mode")))
   :group 'hippie-expand)
@@ -274,12 +273,12 @@ Can contain both regexps matching buffer names (as strings) and major modes
 The expansion functions in `hippie-expand-try-functions-list' are
 tried in order, until a possible expansion is found.  Repeated
 application of `hippie-expand' inserts successively possible
-expansions.  
+expansions.
 With a positive numeric argument, jumps directly to the ARG next
-function in this list.  With a negative argument or just \\[universal-argument], 
-undoes the expansion." 
+function in this list.  With a negative argument or just \\[universal-argument],
+undoes the expansion."
   (interactive "P")
-  (if (or (not arg) 
+  (if (or (not arg)
 	  (and (integerp arg) (> arg 0)))
       (let ((first (or (= he-num -1)
 		       (not (equal this-command last-command)))))
@@ -292,7 +291,7 @@ undoes the expansion."
 	    (setq arg 0))
 	(let ((i (max (+ he-num arg) 0)))
 	  (while (not (or (>= i (length hippie-expand-try-functions-list))
-			  (apply (nth i hippie-expand-try-functions-list) 
+			  (apply (nth i hippie-expand-try-functions-list)
 				 (list (= he-num i)))))
 	    (setq i (1+ i)))
 	  (setq he-num i))
@@ -331,7 +330,7 @@ undoes the expansion."
     (goto-char newpos)))
 
 ;; Substitutes an expansion STR into the correct region (the region
-;; initialized with `he-init-string'). 
+;; initialized with `he-init-string').
 ;; An optional argument TRANS-CASE means that it is ok to transfer case
 ;; from the abbreviation to the expansion if that is possible, and is
 ;; enabled in the buffer.
@@ -382,7 +381,7 @@ undoes the expansion."
 
 
 ;; Check if STR is a member of LST.
-;; Transform to the final case if optional TRANS-CASE is non-NIL.
+;; Transform to the final case if optional TRANS-CASE is non-nil.
 (defun he-string-member (str lst &optional trans-case)
   (if str
       (member (if (and trans-case
@@ -413,14 +412,14 @@ undoes the expansion."
 ;;    (fset 'my-complete-line (make-hippie-expand-function
 ;;                             '(try-expand-line
 ;;                               try-expand-line-all-buffers)))
-;;  
+;;
 ;;;###autoload
 (defmacro make-hippie-expand-function (try-list &optional verbose)
   "Construct a function similar to `hippie-expand'.
 Make it use the expansion functions in TRY-LIST.  An optional second
 argument VERBOSE non-nil makes the function verbose."
   `(function (lambda (arg)
-    ,(concat 
+    ,(concat
       "Try to expand text before point, using the following functions: \n"
       (mapconcat 'prin1-to-string (eval try-list) ", "))
     (interactive "P")
@@ -438,16 +437,16 @@ The argument OLD has to be nil the first call of this function, and t
 for subsequent calls (for further possible completions of the same
 string).  It returns t if a new completion is found, nil otherwise."
   (if (not old)
-      (progn 
+      (progn
 	(he-init-string (he-file-name-beg) (point))
-	(let ((name-part (he-file-name-nondirectory he-search-string))
-	      (dir-part (expand-file-name (or (he-file-name-directory
+	(let ((name-part (file-name-nondirectory he-search-string))
+	      (dir-part (expand-file-name (or (file-name-directory
 					       he-search-string) ""))))
 	  (if (not (he-string-member name-part he-tried-table))
 	      (setq he-tried-table (cons name-part he-tried-table)))
 	  (if (and (not (equal he-search-string ""))
-		   (he-file-directory-p dir-part))
-	      (setq he-expand-list (sort (file-name-all-completions 
+		   (file-directory-p dir-part))
+	      (setq he-expand-list (sort (file-name-all-completions
 					  name-part
 					  dir-part)
 					 'string-lessp))
@@ -461,7 +460,7 @@ string).  It returns t if a new completion is found, nil otherwise."
 	(if old (he-reset-string))
 	())
       (let ((filename (he-concat-directory-file-name
-		       (he-file-name-directory he-search-string)
+		       (file-name-directory he-search-string)
 		       (car he-expand-list))))
 	(he-substitute-string filename)
 	(setq he-tried-table (cons (car he-expand-list) (cdr he-tried-table)))
@@ -471,17 +470,17 @@ string).  It returns t if a new completion is found, nil otherwise."
 (defun try-complete-file-name-partially (old)
   "Try to complete text as a file name, as many characters as unique.
 The argument OLD has to be nil the first call of this function.  It
-returns t if a unique, possibly partial, completion is found, nil 
+returns t if a unique, possibly partial, completion is found, nil
 otherwise."
   (let ((expansion ()))
     (if (not old)
-	(progn 
+	(progn
 	  (he-init-string (he-file-name-beg) (point))
-	  (let ((name-part (he-file-name-nondirectory he-search-string))
-		(dir-part (expand-file-name (or (he-file-name-directory
+	  (let ((name-part (file-name-nondirectory he-search-string))
+		(dir-part (expand-file-name (or (file-name-directory
 						 he-search-string) ""))))
 	    (if (and (not (equal he-search-string ""))
-		     (he-file-directory-p dir-part))
+		     (file-directory-p dir-part))
 		(setq expansion (file-name-completion name-part
 						      dir-part)))
 	    (if (or (eq expansion t)
@@ -494,16 +493,14 @@ otherwise."
 	  (if old (he-reset-string))
 	  ())
 	(let ((filename (he-concat-directory-file-name
-			 (he-file-name-directory he-search-string)
+			 (file-name-directory he-search-string)
 			 expansion)))
 	  (he-substitute-string filename)
 	  (setq he-tried-table (cons expansion (cdr he-tried-table)))
 	  t))))
 
 (defvar he-file-name-chars
-  (cond ((memq system-type '(vax-vms axp-vms))
-	 "-a-zA-Z0-9_/.,~^#$+=:\\[\\]")
-	((memq system-type '(ms-dos windows-nt))
+  (cond ((memq system-type '(ms-dos windows-nt cygwin))
 	 "-a-zA-Z0-9_/.,~^#$+=:\\\\")
 	(t			    ;; More strange file formats ?
 	 "-a-zA-Z0-9_/.,~^#$+="))
@@ -517,63 +514,31 @@ otherwise."
 	  op
 	(point)))))
 
-;; Thanks go to Richard Levitte <levitte@e.kth.se> who helped to make these
-;; work under VMS, and to David Hughes <ukchugd@ukpmr.cs.philips.nl> who 
+;; Thanks go to David Hughes <ukchugd@ukpmr.cs.philips.nl> who
 ;; helped to make it work on PC.
-(defun he-file-name-nondirectory (file)
-  "Fix to make `file-name-nondirectory' work for hippie-expand under VMS."
-  (if (memq system-type '(axp-vms vax-vms))
-      (let ((n (file-name-nondirectory file)))
-	(if (string-match "^\\(\\[.*\\)\\.\\([^\\.]*\\)$" n)
-	    (concat "[." (substring n (match-beginning 2) (match-end 2)))
-	  n))
-    (file-name-nondirectory file)))
-
-(defun he-file-name-directory (file)
-  "Fix to make `file-name-directory' work for hippie-expand under VMS."
-  (if (memq system-type '(axp-vms vax-vms))
-      (let ((n (file-name-nondirectory file))
-	    (d (file-name-directory file)))
-	(if (string-match "^\\(\\[.*\\)\\.\\([^\\.]*\\)$" n)
-	    (concat d (substring n (match-beginning 1) (match-end 1)) "]")
-	  d))
-    (file-name-directory file)))
-
-(defun he-file-directory-p (file)
-  "Fix to make `file-directory-p' work for hippie-expand under VMS."
-  (if (memq system-type '(vax-vms axp-vms))
-      (or (file-directory-p file)
-	  (file-directory-p (concat file "[000000]")))
-    (file-directory-p file)))
-  
 (defun he-concat-directory-file-name (dir-part name-part)
   "Try to slam together two parts of a file specification, system dependently."
   (cond ((null dir-part) name-part)
-	((memq system-type '(axp-vms vax-vms))
-	 (if (and (string= (substring dir-part -1) "]")
-		  (string= (substring name-part 0 2) "[."))
-	     (concat (substring dir-part 0 -1) (substring name-part 1))
-	   (concat dir-part name-part)))
-	((memq system-type '(ms-dos w32))
+	((eq system-type 'ms-dos)
 	 (if (and (string-match "\\\\" dir-part)
 		  (not (string-match "/" dir-part))
 		  (= (aref name-part (1- (length name-part))) ?/))
 	     (aset name-part (1- (length name-part)) ?\\))
 	 (concat dir-part name-part))
-	(t 
+	(t
 	 (concat dir-part name-part))))
-			
+
 (defun try-complete-lisp-symbol (old)
   "Try to complete word as an Emacs Lisp symbol.
 The argument OLD has to be nil the first call of this function, and t
 for subsequent calls (for further possible completions of the same
 string).  It returns t if a new completion is found, nil otherwise."
   (if (not old)
-      (progn 
+      (progn
 	(he-init-string (he-lisp-symbol-beg) (point))
 	(if (not (he-string-member he-search-string he-tried-table))
 	    (setq he-tried-table (cons he-search-string he-tried-table)))
-	(setq he-expand-list 
+	(setq he-expand-list
 	      (and (not (equal he-search-string ""))
 		   (sort (all-completions he-search-string obarray
 					  (function (lambda (sym)
@@ -596,14 +561,14 @@ string).  It returns t if a new completion is found, nil otherwise."
 (defun try-complete-lisp-symbol-partially (old)
   "Try to complete as an Emacs Lisp symbol, as many characters as unique.
 The argument OLD has to be nil the first call of this function.  It
-returns t if a unique, possibly partial, completion is found, nil 
+returns t if a unique, possibly partial, completion is found, nil
 otherwise."
   (let ((expansion ()))
     (if (not old)
-	(progn 
+	(progn
 	  (he-init-string (he-lisp-symbol-beg) (point))
 	  (if (not (string= he-search-string ""))
-	      (setq expansion 
+	      (setq expansion
 		    (try-completion he-search-string obarray
 				    (function (lambda (sym)
 				      (or (boundp sym)
@@ -634,7 +599,7 @@ for subsequent calls (for further possible completions of the same
 string).  It returns t if a new completion is found, nil otherwise."
   (let ((expansion ())
 	(strip-prompt (and (get-buffer-process (current-buffer))
-			   comint-use-prompt-regexp-instead-of-fields
+			   comint-use-prompt-regexp
 			   comint-prompt-regexp)))
     (if (not old)
 	(progn
@@ -649,7 +614,7 @@ string).  It returns t if a new completion is found, nil otherwise."
 		(widen))
 	    ;; Try looking backward unless inhibited.
 	    (if he-search-bw
-		(progn 
+		(progn
 		  (goto-char he-search-loc)
 		  (setq expansion (he-line-search he-search-string
 						  strip-prompt t))
@@ -660,9 +625,9 @@ string).  It returns t if a new completion is found, nil otherwise."
 			(setq he-search-bw ())))))
 
 	    (if (not expansion) ; Then look forward.
-		(progn 
+		(progn
 		  (goto-char he-search-loc)
-		  (setq expansion (he-line-search he-search-string 
+		  (setq expansion (he-line-search he-search-string
 						  strip-prompt nil))
 		  (set-marker he-search-loc (point)))))))
 
@@ -681,7 +646,7 @@ for subsequent calls (for further possible completions of the same
 string).  It returns t if a new completion is found, nil otherwise."
   (let ((expansion ())
 	(strip-prompt (and (get-buffer-process (current-buffer))
-			   comint-use-prompt-regexp-instead-of-fields
+			   comint-use-prompt-regexp
 			   comint-prompt-regexp))
 	(buf (current-buffer))
 	(orig-case-fold-search case-fold-search))
@@ -693,7 +658,7 @@ string).  It returns t if a new completion is found, nil otherwise."
 	  (set-marker he-search-loc 1 (car he-search-bufs))))
 
     (if (not (equal he-search-string ""))
-	(while (and he-search-bufs 
+	(while (and he-search-bufs
 		    (not expansion)
 		    (or (not hippie-expand-max-buffers)
 			(< he-searched-n-bufs hippie-expand-max-buffers)))
@@ -708,9 +673,9 @@ string).  It returns t if a new completion is found, nil otherwise."
 		      (widen))
 		  (goto-char he-search-loc)
 		  (setq strip-prompt (and (get-buffer-process (current-buffer))
-					  comint-use-prompt-regexp-instead-of-fields
+					  comint-use-prompt-regexp
 					  comint-prompt-regexp))
-		  (setq expansion 
+		  (setq expansion
 			(let ((case-fold-search orig-case-fold-search))
 			  (he-line-search he-search-string
 					  strip-prompt nil)))
@@ -732,11 +697,11 @@ string).  It returns t if a new completion is found, nil otherwise."
 	  (he-substitute-string expansion t)
 	  t))))
 
-(defun he-line-search (str strip-prompt reverse) 
+(defun he-line-search (str strip-prompt reverse)
   (let ((result ()))
     (while (and (not result)
 		(if reverse
-		    (re-search-backward 
+		    (re-search-backward
 		     (he-line-search-regexp str strip-prompt)
 		     nil t)
 		    (re-search-forward
@@ -750,7 +715,7 @@ string).  It returns t if a new completion is found, nil otherwise."
 
 (defun he-line-beg (strip-prompt)
   (save-excursion
-    (if (re-search-backward (he-line-search-regexp "" strip-prompt) 
+    (if (re-search-backward (he-line-search-regexp "" strip-prompt)
 			    (save-excursion (beginning-of-line)
 					    (point)) t)
 	(match-beginning 2)
@@ -761,7 +726,7 @@ string).  It returns t if a new completion is found, nil otherwise."
       (concat "\\(" comint-prompt-regexp "\\|^\\s-*\\)\\("
 	      (regexp-quote pat)
 	      "[^\n]*[^ \t\n]\\)")
-      (concat "^\\(\\s-*\\)\\(" 
+      (concat "^\\(\\s-*\\)\\("
 	      (regexp-quote pat)
 	      "[^\n]*[^ \t\n]\\)")))
 
@@ -784,7 +749,7 @@ string).  It returns t if a new completion is found, nil otherwise."
 		(widen))
 	    ;; Try looking backward unless inhibited.
 	    (if he-search-bw
-		(progn 
+		(progn
 		  (goto-char he-search-loc)
 		  (setq expansion (he-list-search he-search-string t))
 		  (set-marker he-search-loc (point))
@@ -794,7 +759,7 @@ string).  It returns t if a new completion is found, nil otherwise."
 			(setq he-search-bw ())))))
 
 	    (if (not expansion) ; Then look forward.
-		(progn 
+		(progn
 		  (goto-char he-search-loc)
 		  (setq expansion (he-list-search he-search-string nil))
 		  (set-marker he-search-loc (point)))))))
@@ -823,7 +788,7 @@ string).  It returns t if a new completion is found, nil otherwise."
 	  (set-marker he-search-loc 1 (car he-search-bufs))))
 
     (if (not (equal he-search-string ""))
-	(while (and he-search-bufs 
+	(while (and he-search-bufs
 		    (not expansion)
 		    (or (not hippie-expand-max-buffers)
 			(< he-searched-n-bufs hippie-expand-max-buffers)))
@@ -837,7 +802,7 @@ string).  It returns t if a new completion is found, nil otherwise."
 		  (if hippie-expand-no-restriction
 		      (widen))
 		  (goto-char he-search-loc)
-		  (setq expansion 
+		  (setq expansion
 			(let ((case-fold-search orig-case-fold-search))
 			  (he-list-search he-search-string nil)))
 		  (set-marker he-search-loc (point))
@@ -858,7 +823,7 @@ string).  It returns t if a new completion is found, nil otherwise."
 	  (he-substitute-string expansion t)
 	  t))))
 
-(defun he-list-search (str reverse) 
+(defun he-list-search (str reverse)
   (let ((result ())
 	beg pos err)
     (while (and (not result)
@@ -872,7 +837,7 @@ string).  It returns t if a new completion is found, nil otherwise."
       (condition-case ()
 	  (forward-list 1)
 	(error (setq err t)))
-      (if (and reverse 
+      (if (and reverse
 	       (> (point) he-string-beg))
 	  (setq err t))
       (if (not err)
@@ -898,13 +863,13 @@ string).  It returns t if a new expansion is found, nil otherwise."
   (if (not old)
       (progn
 	(he-init-string (he-dabbrev-beg) (point))
-	(setq he-expand-list 
+	(setq he-expand-list
 	      (and (not (equal he-search-string ""))
 		   (mapcar (function (lambda (sym)
 			     (if (and (boundp sym) (vectorp (eval sym)))
 				 (abbrev-expansion (downcase he-search-string)
 						   (eval sym)))))
-			   (append '(local-abbrev-table 
+			   (append '(local-abbrev-table
 				     global-abbrev-table)
 				   abbrev-table-name-list))))))
   (while (and he-expand-list
@@ -939,7 +904,7 @@ string).  It returns t if a new expansion is found, nil otherwise."
 		(widen))
 	    ;; Try looking backward unless inhibited.
 	    (if he-search-bw
-		(progn 
+		(progn
 		  (goto-char he-search-loc)
 		  (setq expansion (he-dabbrev-search he-search-string t))
 		  (set-marker he-search-loc (point))
@@ -949,11 +914,11 @@ string).  It returns t if a new expansion is found, nil otherwise."
 			(setq he-search-bw ())))))
 
 	    (if (not expansion) ; Then look forward.
-		(progn 
+		(progn
 		  (goto-char he-search-loc)
 		  (setq expansion (he-dabbrev-search he-search-string nil))
 		  (set-marker he-search-loc (point)))))))
-	
+
     (if (not expansion)
 	(progn
 	  (if old (he-reset-string))
@@ -963,7 +928,7 @@ string).  It returns t if a new expansion is found, nil otherwise."
 	  t))))
 
 (defun try-expand-dabbrev-all-buffers (old)
-  "Tries to expand word \"dynamically\", searching all other buffers.
+  "Try to expand word \"dynamically\", searching all other buffers.
 The argument OLD has to be nil the first call of this function, and t
 for subsequent calls (for further possible expansions of the same
 string).  It returns t if a new expansion is found, nil otherwise."
@@ -978,7 +943,7 @@ string).  It returns t if a new expansion is found, nil otherwise."
 	  (set-marker he-search-loc 1 (car he-search-bufs))))
 
     (if (not (equal he-search-string ""))
-	(while (and he-search-bufs 
+	(while (and he-search-bufs
 		    (not expansion)
 		    (or (not hippie-expand-max-buffers)
 			(< he-searched-n-bufs hippie-expand-max-buffers)))
@@ -1021,40 +986,37 @@ The argument OLD has to be nil the first call of this function, and t
 for subsequent calls (for further possible expansions of the same
 string).  It returns t if a new expansion is found, nil otherwise."
   (let ((expansion ())
-	(buf (current-buffer))
 	(flag (if (frame-visible-p (window-frame (selected-window)))
 		  'visible t)))
-    (if (not old)
-	(progn
-	   (he-init-string (he-dabbrev-beg) (point))
-	   (setq he-search-window (selected-window))
-	   (set-marker he-search-loc
-		       (window-start he-search-window)
-		       (window-buffer he-search-window))))
+    (unless old
+      (he-init-string (he-dabbrev-beg) (point))
+      (setq he-search-window (selected-window))
+      (set-marker he-search-loc
+                  (window-start he-search-window)
+                  (window-buffer he-search-window)))
 
     (while (and (not (equal he-search-string ""))
-		(marker-position he-search-loc)
-		(not expansion))
-      (save-excursion
-	(set-buffer (marker-buffer he-search-loc))
-	(goto-char he-search-loc)
-	(setq expansion (he-dabbrev-search he-search-string ()
-					   (window-end he-search-window)))
-	(if (and expansion
-		 (eq (marker-buffer he-string-beg) (current-buffer))
-		 (eq (marker-position he-string-beg) (match-beginning 0)))
-	    (setq expansion (he-dabbrev-search he-search-string ()
-					       (window-end he-search-window))))
-	(set-marker he-search-loc (point) (current-buffer)))
-      (if (not expansion)
-	  (progn 
-	    (setq he-search-window (next-window he-search-window nil flag))
-	    (if (eq he-search-window (selected-window))
-		(set-marker he-search-loc nil)
-	      (set-marker he-search-loc (window-start he-search-window)
-			  (window-buffer he-search-window))))))
-    
-    (set-buffer buf)
+                (marker-position he-search-loc)
+                (not expansion))
+      (with-current-buffer (marker-buffer he-search-loc)
+        (save-excursion
+          (goto-char he-search-loc)
+          (setq expansion (he-dabbrev-search he-search-string ()
+                                             (window-end he-search-window)))
+          (if (and expansion
+                   (eq (marker-buffer he-string-beg) (current-buffer))
+                   (eq (marker-position he-string-beg) (match-beginning 0)))
+              (setq expansion
+                    (he-dabbrev-search he-search-string ()
+                                       (window-end he-search-window))))
+          (set-marker he-search-loc (point) (current-buffer))))
+      (unless expansion
+        (setq he-search-window (next-window he-search-window nil flag))
+        (if (eq he-search-window (selected-window))
+            (set-marker he-search-loc nil)
+          (set-marker he-search-loc (window-start he-search-window)
+                      (window-buffer he-search-window)))))
+
     (if (not expansion)
 	(progn
 	  (if old (he-reset-string))
@@ -1072,7 +1034,7 @@ string).  It returns t if a new expansion is found, nil otherwise."
 		      (t
 		       (concat "\\<" (regexp-quote pattern)
 			       "\\(\\sw\\|\\s_\\)+")))))
-    (while (and (not result) 
+    (while (and (not result)
 		(if reverse
 		     (re-search-backward regpat limit t)
 		     (re-search-forward regpat limit t)))
@@ -1104,7 +1066,7 @@ for subsequent calls (for further possible completions of the same
 string).  It returns t if a new completion is found, nil otherwise."
   (let ((expansion ()))
     (if (not old)
-	(progn 
+	(progn
 	  (he-init-string (he-dabbrev-beg) (point))
 	  (setq he-expand-list
 		(if (not (equal he-search-string ""))
@@ -1130,7 +1092,7 @@ string).  It returns t if a new completion is found, nil otherwise."
 		       (concat "\\<" (regexp-quote pattern)
 			       "\\(\\sw\\|\\s_\\)+"))))
 	(killstr (car he-expand-list)))
-    (while (and (not result) 
+    (while (and (not result)
 		he-expand-list)
       (while (and (not result)
 		  (string-match regpat killstr he-search-loc2))
@@ -1143,10 +1105,10 @@ string).  It returns t if a new completion is found, nil otherwise."
 			   '(?_ ?w)))
 		(he-string-member result he-tried-table t))
 	    (setq result nil)))     ; ignore if bad prefix or already in table
-      (if (and (not result) 
+      (if (and (not result)
 		he-expand-list)
 	  (progn
-	    (setq he-expand-list (cdr he-expand-list)) 
+	    (setq he-expand-list (cdr he-expand-list))
 	    (setq killstr (car he-expand-list))
 	    (setq he-search-loc2 0))))
     result))
@@ -1158,11 +1120,11 @@ for subsequent calls (for further possible completions of the same
 string).  It returns t if a new completion is found, nil otherwise."
   (let ((expansion ()))
     (if (not old)
-	(progn 
+	(progn
 	  (he-init-string (he-kill-beg) (point))
 	  (if (not (he-string-member he-search-string he-tried-table))
 	      (setq he-tried-table (cons he-search-string he-tried-table)))
-	  (setq he-expand-list 
+	  (setq he-expand-list
 		(if (not (equal he-search-string ""))
 		    kill-ring))
 	  (setq he-search-loc2 ())))
@@ -1206,7 +1168,7 @@ string).  It returns t if a new completion is found, nil otherwise."
       (if (and (not result)
 		he-expand-list)
 	  (progn
-	    (setq he-expand-list (cdr he-expand-list)) 
+	    (setq he-expand-list (cdr he-expand-list))
 	    (setq killstr (car he-expand-list))
 	    (setq pos -1))))
     result))
@@ -1222,4 +1184,5 @@ string).  It returns t if a new completion is found, nil otherwise."
 
 (provide 'hippie-exp)
 
+;; arch-tag: 5e6e00bf-b061-4a7a-9b46-de0ae105ab99
 ;;; hippie-exp.el ends here
